# Download dae_basic
mkdir -p dae-factuality/dae_basic
gdown https://drive.google.com/uc?id=17j5PWBMj0bHp3csIhGuJPOJbYz-C2th0 --output dae-factuality/dae_basic/config.json
gdown https://drive.google.com/uc?id=1CxKHG_4ejlrWVtdeLhvWcFmmA8_MLvBp --output dae-factuality/dae_basic/pytorch_model.bin
gdown https://drive.google.com/uc?id=1zYzW8NKD4UJb0I9vDRHzxs0oRa975dDq --output dae-factuality/dae_basic/special_tokens_map.json
gdown https://drive.google.com/uc?id=1iP6E91Z2BF-5amraJX4hDfU_Kfyt5yhv --output dae-factuality/dae_basic/tokenizer_config.json
gdown https://drive.google.com/uc?id=1fuFUeFirS_QSeH2apa8BHktKyiqheX51 --output dae-factuality/dae_basic/training_args.bin
gdown https://drive.google.com/uc?id=1xeG14QmcUjjqzPhRX_yU90X7XAyABinA --output dae-factuality/dae_basic/vocab.txt

# Download dae_w_syn
mkdir -p dae-factuality/dae_w_syn
gdown https://drive.google.com/uc?id=1nozkUmEiJmob5nhRKSURpvJalQ2jHBqf --output dae-factuality/dae_w_syn/config.json
gdown https://drive.google.com/uc?id=1m70_VLap1htZtc7mdBSKsGkd7tnfcVbx --output dae-factuality/dae_w_syn/pytorch_model.bin
gdown https://drive.google.com/uc?id=1h2cJZ1YHCGCiWDKlxjMpPfcudLABwoeS --output dae-factuality/dae_w_syn/special_tokens_map.json
gdown https://drive.google.com/uc?id=15QExDrhScIlEgj2usPgj-bHo9R4gJ6oG --output dae-factuality/dae_w_syn/tokenizer_config.json
gdown https://drive.google.com/uc?id=1Pl5yKgOIC38k6BSwrDrCV3s73WMcYUav --output dae-factuality/dae_w_syn/training_args.bin
gdown https://drive.google.com/uc?id=1C41uGYYCsZBTLt_bzgvIL3KpO1giAF0H --output dae-factuality/dae_w_syn/vocab.txt

# Download dae_w_syn_hallu
mkdir -p dae-factuality/dae_w_syn_hallu
gdown https://drive.google.com/uc?id=1FiRqtXql-_zp3y68A1sBcM0KIuePL3SE --output dae-factuality/dae_w_syn_hallu/config.json
gdown https://drive.google.com/uc?id=1NrYPrITpe4UCq1J36xiRrVVL1envWb9y --output dae-factuality/dae_w_syn_hallu/pytorch_model.bin
gdown https://drive.google.com/uc?id=1QFJft-zuN02Msq9QScbKn0AutV-iXbSr --output dae-factuality/dae_w_syn_hallu/special_tokens_map.json
gdown https://drive.google.com/uc?id=1BkiYuB-5q8G0a2r3e9OS3CvmjFd-GKpO --output dae-factuality/dae_w_syn_hallu/tokenizer_config.json
gdown https://drive.google.com/uc?id=1RaNSqKErBGKaWYiJGX2Zm1nvQ7R8jxnM --output dae-factuality/dae_w_syn_hallu/training_args.bin
gdown https://drive.google.com/uc?id=1W3bO2FFFle_fSnL1rRqcuVDWqJCWEQLE --output dae-factuality/dae_w_syn_hallu/vocab.txt